#!/usr/bin/perl -w

package Printing;

use strict;

use Encode;
use utf8;
#use open ':utf8', ':std';

use POSIX qw(strftime);

use lib "./";
use lib::Dialog;

my %MyMoney;


sub new {
  my $class = shift;
  my $self = {};
  bless $self, $class;
  %MyMoney=@_;
  return $self;
};

sub ParseToLatex {
    my($self,$txt)=@_;
    $txt=~s/&/\\&/g;
    $txt=~s/_/\\_/g;
    return($txt);
};

sub print_table {
    my $self=shift;
    my (%print_table)=@_;

    #
    # hlavicka tabulky
    #
    my $header="";
    my $format="";
    foreach $_ (@{$print_table{'data_header'}}) {
        if($header) {
            $header.=" & ";
        };
        $header.="\\footnotesize \\bfseries ".$self->ParseToLatex($_);
        $format.="l";
    };

    #
    # radky tabulky
    #
    my $rows="";
    foreach my $rowhash (@{$print_table{'data'}}) {
        my $row="";
        foreach $_ (@{$rowhash}) {
            if($row) {
                $row.=" & ";
            };
            $row.="\\footnotesize ". $self->ParseToLatex($_);
        };
        $rows.=$row." \\\\\n";
    };

    my $table="\\begin{longtable}{".$format."}\n";
    $table.=$header." \\\\\\hline \\endhead\n";
    $table.=$rows;
    $table.="\\end{longtable}";
    #
    # nacist template a parsovat klice
    #
    my $output="";
    open(FILE,"./data/printing_templates/simple_table.latex");
    while(<FILE>) {
        $output.=Encode::decode('utf8',$_);
    };
    close(FILE);

    my $keyval; 
    $keyval=$print_table{'title'};
    $output=~s/KEYDOCUMENTTITLE/${keyval}/g;
    $keyval=$MyMoney{'AgendaData'}->get('ob_nazev');
    $output=~s/KEYSUBJECTNAME/${keyval}/g;
    $keyval=$MyMoney{'AgendaData'}->get('ico');
    $output=~s/KEYICO/${keyval}/g;
    $keyval=$MyMoney{'RokData'}->get('ucetni_rok');
    $output=~s/KEYYEAR/${keyval}/g;
    $keyval=$MyMoney{'Uzivatel'}->get('jmeno');
    if($keyval eq '-') {
        $keyval="";
    };
    $output=~s/KEYUSERNAME/${keyval}/g;
    $output=~s/KEYTABLEDATA/${table}/g;
    
    #
    # vyrobit .latex a .dvi
    #
    my $outfilename="/tmp/simple_table_$$";
    open(FILE,">".$outfilename.".latex");
    print(FILE Encode::encode('iso-8859-2',$output));
    close(FILE);
    `cd /tmp/;/usr/bin/latex -interaction=nonstopmode ${outfilename}.latex`;
    `cd /tmp/;/usr/bin/latex -interaction=nonstopmode ${outfilename}.latex`;
    

    #
    # zobrazit nahled
    #
    my $pid=$$;
    if(fork) {
        return;
    };
    my $newpid=$$;
    if($pid ne $newpid) {
        system("/usr/bin/kdvi ${outfilename}.dvi");
        unlink("${outfilename}.aux");
        unlink("${outfilename}.dvi");
        unlink("${outfilename}.latex");
        unlink("${outfilename}.log");
        exit;
    } else {
        print("Error: Printing->print_table() - nepodarilo se volani fork()!\n");
    };
};

sub print_document {
    my $self=shift;
    my $template=shift;
    my @PrintDocuments=@_;
    
    my $tmpdir="/tmp/";
    my $printkey=$$."_".strftime('%s',localtime);

    my @files=();    
    my $i=0;
    foreach my $prt (@PrintDocuments) {
        my $RawLatexDefs=shift(@{$prt});
        my %PrintHash=@{$prt};        

        my $datadef="";
        foreach $_ (keys(%PrintHash)) {
            $datadef.="\\def\\$_\{".$self->ParseToLatex($PrintHash{$_})."\}\n";
        };
    
        my $output="";
        open(FILE,"./data/printing_templates/".$template);
        while(<FILE>) {
            if(/^%DATADEFINITIONS%/) {
                $output.=$datadef."\n".$RawLatexDefs."\n\\def\\tuxlogo{".$MyMoney{'MyMoneyDIR'}."/data/printing_templates/tuxsit.eps}\n";
            } else {
                $output.=Encode::decode('iso-8859-2',$_);
            };
        };
        close(FILE);
    
        #
        # vyrobit .tex
        #
        my $outfilename=$tmpdir."/".$template."_".$printkey."_".$i;
        open(FILE,">".$outfilename.".tex");
        print(FILE Encode::encode('iso-8859-2',$output));
        close(FILE);
        push(@files,$outfilename.".tex");
        $i++;
    };

    # vyrobit root_document
    my $includefiles="";
    foreach $_ (@files) {
        $includefiles.="\\includefile{".$_."}\n";
    };
    open(FILE,"./data/printing_templates/root_document.tex");
    my $output;
    while(<FILE>) {
        if(/^%INCLUDEFILES%/) {
            $output.=$includefiles;
        } else {
            $output.=Encode::decode('iso-8859-2',$_);
        };
    };
    close(FILE);

    my $outfilename=$tmpdir."/".$template."_".$printkey."_".$i."_root_document";
    open(FILE,">".$outfilename.".tex");
    print(FILE Encode::encode('iso-8859-2',$output));
    close(FILE);
    
    # vyrobit dvi
    `cd ${tmpdir};/usr/bin/latex -interaction=nonstopmode ${outfilename}.tex`;
    

    #
    # zobrazit nahled
    #
    my $pid=$$;
    if(fork) {
        return;
    };
    my $newpid=$$;
    if($pid ne $newpid) {
        system("/usr/bin/kdvi ${outfilename}.dvi");
        unlink("${outfilename}.aux");
        unlink("${outfilename}.dvi");
        unlink("${outfilename}.tex");
        unlink("${outfilename}.log");
        foreach $_ (@files) {
            unlink($_);
        };
        exit;
    } else {
        print("Error: Printing->print_table() - nepodarilo se volani fork()!\n");
    };
};

1;
